*=$1490
        ldx     #0
scrlp0
        lda     #32
        sta     $1E00,x
        sta     $1F00,x
        lda     #0
        sta     $9600,x
        sta     $9700,x
        inx
        bne     scrlp0

        jsr $e581       ;home
        jsr $e8c3       ;linefeed
        lda #<itxt1
        ldy #>itxt1
        jsr $cb1e       ;text out

        lda #$7f
        sta $912e     ; disable and acknowledge interrupts
        sta $912d

ycora   ldx #130  ; wait for this raster line (times 2) until we start first irq look.
scresync
        cpx $9004
        bne scresync
        ldy #9
        bit $24
scresync2
        ldx $9004
        txa
        bit $24
;#if SYSTEM & PAL
        ldx #24
;#endif
;#if SYSTEM & NTSC
        ;bit $24
        ;ldx #21
;#endif
        dex
        bne *-1       ; first spend some time (so that the whole
        cmp $9004     ; loop will be 2 raster lines)
        bcs *+2       ; save one cycle if $9004 changed too late
        dey
        bne scresync2

timers
        lda #$40      ; enable Timer A free run of both VIAs
        sta $911b
        sta $912b

        lda timerv      ;length of timer low byte
        ldx timerv+1    ; length of timer high byte
        sta $9116     ; load the timer 1 low byte
        sta $9126     ; load the timer low byte counter

        stx $9125     ; start the IRQ timer A (high byte latch)
        ldy #10       ; spend some time
        dey           ; before starting the reference timer
        bne *-1
        stx $9115     ; start the reference timer

pointers
        lda #<irqstart     ; set the raster IRQ routine pointer
        sta $314
        lda #>irqstart
        sta $315
        lda #$c0
        sta $912e     ; enable Timer A underflow interrupts

        rts

itxt1
        text "morse code keyboard!  ",0



irqstart
        lda     $9111
        AND     #32
        CMP     #32
        BEQ     notfire
        ;FIRE.. do things here.
        lda     #251
        sta     $900a       ;800Hz tone
        lda     $900e
        and     #240
        ora      #15
        sta     $900e   ;sound on

        lda     timer
        CMP     #$ff
        BEQ     noinct  ; fire pressed for a long time, no increment
        INC     timer   ; number of 50/th of seconds
noinct
        JMP     $eabf     ; return to normal IRQ
notfire

        lda     timer
        CMP     #0
        bne     joydep  ; timer is not zero, so fire had been pressed
        jmp     noaction
        ; joystick was depressed, so look at timing to determine code
joydep
        sta     $3
        lda     $900e
        and     #240
        ;ora      #15
        sta     $900e   ;sound off
        lda     #0
        sta     $900a       ;no tone
        ldx     #0
        stx     timer           ; reset timer
        lda     $3
        cmp     #10
        BMI     shorttime
        JMP     longtime
        ; tap was shorter than 1/2th second
shorttime
        ldx     morsepoint      ;pointer to morse stack
        lda     #1              ;short morse point
        sta     morsecode,x
        lda     #170
        sta     $1E00,x
        inx
        stx     morsepoint      ;next


        ;sta     mtxt
        ;lda     #<mtxt
        ;ldy     #>mtxt
        ;jsr     $cb1e           ; print a point

        ldx     morsepoint

        cpx     #5              ; after 4 points, wait
        bne     nofinishm
morsefinish
        ; finished one code (4 points), so print out character
        ldx     #0
        stx     $2
morseloop1
        ldy     #0
morseloop2
        lda     alpabet,x
        cmp     morsecode,y     ; search database for character
        bne     notcorrect
        inx
        iny
        cpy     #4
        bne     morseloop2
        ; got here because we have a match!
        ; index in $2
        lda     $2
        lsr
        lsr
        clc
        adc     #$41    ;="A"+index
        sta     mtxt
        sta     mlast
        sei
        ldx     $c6
        cpx     $0289
        bcs     nokey1  ;keyboard buffer full
        sta     $0277,x  ;store character
        inx
        stx     $c6     ;store increased keyboard buffer
        cli
nokey1
        ;lda     #<mtxt
        ;ldy     #>mtxt
        ;jsr     $cb1e
mreset
        lda     #0
        sta     timer
        sta     ntimer
        sta     morsepoint      ;reset
        sta     morsecode
        sta     morsecode+1
        sta     morsecode+2
        sta     morsecode+3
        lda     #" "
        sta     $1E00
        sta     $1E01
        sta     $1E02
        sta     $1E03
        jmp     exit
notcorrect
        lda     $2
        clc
        adc     #4
        sta     $2
        tax                ;next morse code

        lda     alpabet,x       ;at end?
        cmp     #255
        bne     morseloop1
        ; at end so reset (since not found) 
        jmp     mreset
nofinishm
        ldx     #0
        stx     ntimer
        stx     timer
        jmp     nextcycle
longtime
        ; tap was longer than 10/50th second
        ldx     morsepoint
        lda     #2
        sta     morsecode,x
        lda     #173
        sta     $1E00,x
        inx
        stx     morsepoint

        ;sta     mtxt
        ;lda     #<mtxt
        ;ldy     #>mtxt
        ;jsr     $cb1e           ; print a streak

        ldx     morsepoint
        cpx     #5
        bne     nofinishm
        jmp     morsefinish

nextcycle
        LDA     #0
        STA     timer
noaction
        ;nothing happened, so check time since last happening
        inc     ntimer
        lda     ntimer
        cmp     #25    ; 1 second since last fire?
        bne     exit
   ;     ldx     morsepoint
   ;     lda     #0
endlp
   ;     sta     morsecode,x     ; fill remaining bytes with 0
   ;     inx
   ;     cpx     #4
   ;     bne     endlp
   ;     stx     morsepoint
        lda     morsepoint
        cmp     #0
        beq     space
        jmp     morsefinish
space
        lda     #32
        cmp     mlast
        beq     exit            ;not two spaces in a row
        sta     mtxt
        sta     mlast
        sei
        ldx     $c6
        cpx     $0289
        bcs     nokey2  ;keyboard buffer full
        inx
        ;sta     $0277,x  ;store character
        ;stx     $c6     ;store increased keyboard buffer
        cli
nokey2
        ;lda     #<mtxt
        ;ldy     #>mtxt
        ;jsr     $cb1e
exit
        ldx     #65
        ldy     #21
scrlp1
        lda     itxt1,y
        clc
        adc     #$40
        cpx     #44
        bpl     scrtg1
        lda     #32
        cpx     #4
        bpl     scrtg1
        lda     morsecode,x
        clc
        adc     #32
        cmp     #32
        beq     scrtg1
        cmp     #33
        beq     scrtg0
        lda     #173
        bne     scrtg1
scrtg0
        lda     #170
scrtg1
        sta     $1E00,x
        dey
        dex
        bpl     scrlp1

        JMP     $eabf     ; return to normal IRQ

timer   byte    0
ntimer  byte    0
morsepoint
        byte    0
morsecode
        byte    0,0,0,0,0
mtxt    byte    0,0
mlast   byte    32

alpabet ;1=short, 2=long, 0=nothing
        byte    1,2,0,0         ;A
        byte    2,1,1,1         ;B
        byte    2,1,2,1         ;C
        byte    2,1,1,0         ;D
        byte    1,0,0,0         ;E
        byte    1,1,2,1         ;F
        byte    2,2,1,0         ;G
        byte    1,1,1,1         ;H
        byte    1,1,0,0         ;I
        byte    1,2,2,2         ;J
        byte    2,1,2,0         ;K
        byte    1,2,1,1         ;L
        byte    2,2,0,0         ;M
        byte    2,1,0,0         ;N
        byte    2,2,2,0         ;O
        byte    1,2,2,1         ;P
        byte    2,2,1,2         ;Q
        byte    1,2,1,0         ;R
        byte    1,1,1,0         ;S
        byte    2,0,0,0         ;T
        byte    1,1,2,0         ;U
        byte    1,1,1,2         ;V
        byte    1,2,2,0         ;W
        byte    2,1,1,2         ;X
        byte    2,1,2,2         ;Y
        byte    2,2,1,1         ;Z
        byte    255
;LINES=312
;CYCLES=71
;TIMERV = LINES * CYCLES - 2

timerv word 22150       ;once per screen update



